@@ -2,7 +2,8 @@ import sys
 
 import matplotlib
 import matplotlib.style as mplstyle
-mplstyle.use('fast')
+
+mplstyle.use("fast")
 
 matplotlib.use("Agg")
 
@@ -17,42 +18,44 @@ matplotlib.rc("legend", fontsize=size)  # legend fontsize
 matplotlib.rc("figure", titlesize=size)  # fontsize of the figure title
 matplotlib.rc("figure", autolayout=True)
 
+import itertools
+import logging
+
+# parallelization
+import multiprocessing
+import os
+import warnings
+from pathlib import Path
+
+import corner
+import numpy
+import pandas
+import scipy.interpolate
+import seaborn as sns
+from addict import Dict
+from cadet import H5
 from matplotlib import figure
 from matplotlib.backends.backend_agg import FigureCanvasAgg as FigureCanvas
 from mpl_toolkits.mplot3d import Axes3D
 
+import CADETMatch.loggerwriter as loggerwriter
+import CADETMatch.util as util
 from CADETMatch.cache import cache
 
-from pathlib import Path
-import pandas
-import numpy
-import scipy.interpolate
-import itertools
-
-from cadet import H5
-from addict import Dict
-
-# parallelization
-import multiprocessing
-
-import os
-import warnings
-import corner
-import seaborn as sns
-import CADETMatch.util as util
-import logging
-import CADETMatch.loggerwriter as loggerwriter
 
 def new_range(flat_chain):
     lb_data, mid_data, ub_data = numpy.percentile(flat_chain, [5, 50, 95], 0)
 
-    distance = numpy.max(numpy.abs(numpy.array([lb_data - mid_data, ub_data - mid_data])), axis=0)
+    distance = numpy.max(
+        numpy.abs(numpy.array([lb_data - mid_data, ub_data - mid_data])), axis=0
+    )
 
     lb_range = mid_data - 2 * distance
     ub_range = mid_data + 2 * distance
 
     return numpy.array([lb_range, ub_range])
 
+
 def main():
     cache.setup_dir(sys.argv[1])
     util.setupLog(cache.settings["resultsDirLog"], "graph_corner.log")
@@ -60,7 +63,28 @@ def main():
 
     multiprocessing.get_logger().info("graphing directory %s", os.getcwd())
 
-    graphCorner(cache)
+    resultDir = Path(cache.settings["resultsDirBase"])
+    result_lock = resultDir / "result.lock"
+
+
+    mcmcDir = Path(cache.settings["resultsDirMCMC"])
+    mcmc_h5 = mcmcDir / "mcmc.h5"
+    result_h5 = resultDir / "result.h5"
+
+    if mcmc_h5.exists():
+        data_mcmc = H5()
+        data_mcmc.filename = mcmc_h5.as_posix()
+        data_mcmc.load(lock=True)
+
+        data_results = None
+    else:
+        data_results = H5()
+        data_results.filename = result_h5.as_posix()
+        data_results.load(lock=True)
+
+        data_mcmc = None
+
+    graphCorner(cache, data_mcmc, data_results)
 
 
 def plotChain(flat_chain, flat_chain_transform, headers, out_dir, prefix):
@@ -73,6 +97,9 @@ def plotChain(flat_chain, flat_chain_transform, headers, out_dir, prefix):
         chain = flat_chain
         chain_transform = flat_chain_transform
 
+    if len(chain) == 0:
+        return
+
     # stat corner plot
     fig_size = 6 * len(headers)
     fig = corner.corner(
@@ -114,22 +141,45 @@ def plotMCMCParam(out_dir, param, chain, header):
     graph.plot(x, chain[0, :, param], "r", label="5")
     graph.plot(x, chain[1, :, param], "k", label="mean")
     graph.plot(x, chain[2, :, param], "r", label="95")
-    graph.fill_between(x, chain[0, :, param], chain[2, :, param], facecolor="r", alpha=0.5)
+    graph.fill_between(
+        x, chain[0, :, param], chain[2, :, param], facecolor="r", alpha=0.5
+    )
     graph.legend()
     graph.set_title(header)
     fig.set_size_inches((12, 12))
-    fig.savefig(str(out_dir / ("%s.png" % header)))
+    print(header, header.replace("/", "_"))
+    fig.savefig(str(out_dir / ("%s.png" % header.replace("/", "_"))))
 
 
 def plotMCMCVars(out_dir, headers, data):
     for param_idx, header in enumerate(headers):
         if "train_chain_stat" in data.root:
-            plotMCMCParam(out_dir, param_idx, data.root.train_chain_stat, "Train " + header + str(param_idx))
-            plotMCMCParam(out_dir, param_idx, data.root.train_chain_stat_transform, "Train " + header + " Transform" + str(param_idx))
+            plotMCMCParam(
+                out_dir,
+                param_idx,
+                data.root.train_chain_stat,
+                "Train " + header + str(param_idx),
+            )
+            plotMCMCParam(
+                out_dir,
+                param_idx,
+                data.root.train_chain_stat_transform,
+                "Train " + header + " Transform" + str(param_idx),
+            )
 
         if "run_chain_stat" in data.root:
-            plotMCMCParam(out_dir, param_idx, data.root.run_chain_stat, "Run " + header + str(param_idx))
-            plotMCMCParam(out_dir, param_idx, data.root.run_chain_stat_transform, "Run " + header + " Transform" + str(param_idx))
+            plotMCMCParam(
+                out_dir,
+                param_idx,
+                data.root.run_chain_stat,
+                "Run " + header + str(param_idx),
+            )
+            plotMCMCParam(
+                out_dir,
+                param_idx,
+                data.root.run_chain_stat_transform,
+                "Run " + header + " Transform" + str(param_idx),
+            )
 
 
 def keep_header(char):
@@ -145,21 +195,13 @@ def clean_header(header):
     return " ".join(temp)
 
 
-def graphCorner(cache):
+def graphCorner(cache, data_mcmc=None, data_results=None):
     multiprocessing.get_logger().info("plotting corner plots")
     headers = list(cache.parameter_headers_actual)
     headers = [clean_header(header) for header in headers]
 
-    resultDir = Path(cache.settings["resultsDir"])
-    result_h5 = resultDir / "result.h5"
-
-    miscDir = Path(cache.settings["resultsDirMCMC"])
-    mcmc_h5 = miscDir / "mcmc.h5"
-
-    if mcmc_h5.exists():
-        data = H5()
-        data.filename = mcmc_h5.as_posix()
-        data.load()
+    if data_mcmc is not None:
+        data = data_mcmc
 
         out_dir = cache.settings["resultsDirProgress"]
 
@@ -179,7 +221,13 @@ def graphCorner(cache):
 
         if "flat_chain" in data.root:
             multiprocessing.get_logger().info("plot flat chain")
-            plotChain(data.root.flat_chain, data.root.flat_chain_transform, headers, out_dir, "full")
+            plotChain(
+                data.root.flat_chain,
+                data.root.flat_chain_transform,
+                headers,
+                out_dir,
+                "full",
+            )
 
         if "burn_in_acceptance" in data.root:
             multiprocessing.get_logger().info("plot burn in acceptance")
@@ -237,16 +285,22 @@ def graphCorner(cache):
             fig.set_size_inches((12, 12))
             fig.savefig(str(out_dir / "mcmc_acceptance.png"))
 
-        plotChain(data.root.train_flat_chain, data.root.train_flat_chain_transform, headers, out_dir, "train")
+        plotChain(
+            data.root.train_flat_chain,
+            data.root.train_flat_chain_transform,
+            headers,
+            out_dir,
+            "train",
+        )
         plotMCMCVars(out_dir, headers, data)
 
     else:
-        data = H5()
-        data.filename = result_h5.as_posix()
-        data.load()
+        data = data_results
 
         if len(data.root.input) > 1e5:
-            indexes = numpy.random.choice(data.root.input.shape[0], int(1e5), replace=False)
+            indexes = numpy.random.choice(
+                data.root.input.shape[0], int(1e5), replace=False
+            )
             data_input = data.root.input[indexes]
             data_input_transform = data.root.input_transform[indexes]
             weight_min = data.root.output_meta[indexes, 1]
@@ -263,7 +317,9 @@ def graphCorner(cache):
         acceptable = max_scores > 0.01
 
         if numpy.any(acceptable):
-            multiprocessing.get_logger().info("graphing remove %s points", len(max_scores) - numpy.sum(acceptable))
+            multiprocessing.get_logger().info(
+                "graphing remove %s points", len(max_scores) - numpy.sum(acceptable)
+            )
 
             data_input = data_input[acceptable]
             data_input_transform = data_input_transform[acceptable]
@@ -273,18 +329,38 @@ def graphCorner(cache):
         out_dir = cache.settings["resultsDirProgress"]
 
         create_corner(out_dir, "corner.png", headers, data_input, weights=None)
-        create_corner(out_dir, "corner_min.png", headers, data_input, weights=weight_min)
-        create_corner(out_dir, "corner_prod.png", headers, data_input, weights=weight_prod)
+        create_corner(
+            out_dir, "corner_min.png", headers, data_input, weights=weight_min
+        )
+        create_corner(
+            out_dir, "corner_prod.png", headers, data_input, weights=weight_prod
+        )
 
         # transformed entries
-        create_corner(out_dir, "corner_transform.png", headers, data_input_transform, weights=None)
-        create_corner(out_dir, "corner_min_transform.png", headers, data_input_transform, weights=weight_min)
-        create_corner(out_dir, "corner_prod_transform.png", headers, data_input_transform, weights=weight_prod)
+        create_corner(
+            out_dir, "corner_transform.png", headers, data_input_transform, weights=None
+        )
+        create_corner(
+            out_dir,
+            "corner_min_transform.png",
+            headers,
+            data_input_transform,
+            weights=weight_min,
+        )
+        create_corner(
+            out_dir,
+            "corner_prod_transform.png",
+            headers,
+            data_input_transform,
+            weights=weight_prod,
+        )
 
 
 def create_corner(dir, filename, headers, data, weights=None):
     if len(data) <= len(headers):
-        multiprocessing.get_logger().info("There is not enough data to generate corner plots")
+        multiprocessing.get_logger().info(
+            "There is not enough data to generate corner plots"
+        )
         return
     if numpy.all(numpy.min(data, 0) < numpy.max(data, 0)):
         if weights is None or numpy.max(weights) > numpy.min(weights):
