@@ -7,41 +7,34 @@
 # Base noise
 # Signal noise
 
-import itertools
-import CADETMatch.score as score
-import CADETMatch.kde_util as kde_util
-from cadet import Cadet, H5
-from pathlib import Path
-import numpy
-import multiprocessing
-from sklearn.neighbors import KernelDensity
-from sklearn import preprocessing
 import copy
+import itertools
+import multiprocessing
+from pathlib import Path
+
+import numpy
+from cadet import H5, Cadet
+from sklearn import preprocessing
+from sklearn.neighbors import KernelDensity
+
+import CADETMatch.kde_util as kde_util
+import CADETMatch.score as score
 
 bw_tol = 1e-4
 
+import sys
 import warnings
 
-with warnings.catch_warnings():
-    warnings.filterwarnings("ignore", category=FutureWarning)
-    import h5py
-
-import scipy.optimize
-
-import CADETMatch.util as util
-import CADETMatch.cache as cache
-
-import CADETMatch.synthetic_error as synthetic_error
-
 import joblib
-import subprocess
-import sys
-
 import SALib.sample.sobol_sequence
+import scipy.optimize
 import scipy.stats
 
-atol = 1e-3
-rtol = 1e-3
+import CADETMatch.cache as cache
+import CADETMatch.synthetic_error as synthetic_error
+import CADETMatch.util as util
+import CADETMatch.sub as sub
+
 
 def mirror(data):
     data_max = numpy.max(data, 0)
@@ -56,7 +49,10 @@ def mirror(data):
     data_mirror = numpy.zeros(data.shape)
 
     data_mirror[:, mirror_index] = (
-        data_max[mirror_index] + data_max[mirror_index] - numpy.copy(data[:, mirror_index]) + min_value[mirror_index]
+        data_max[mirror_index]
+        + data_max[mirror_index]
+        - numpy.copy(data[:, mirror_index])
+        + min_value[mirror_index]
     )
     data_mirror[:, keep_index] = data[:, keep_index]
     full_data = numpy.vstack([data_mirror, data])
@@ -75,7 +71,7 @@ def setupKDE(cache):
 
     scores_scaler = scaler.transform(scores_mirror)
 
-    kde = KernelDensity(kernel='gaussian', atol=atol, rtol=rtol)
+    kde = KernelDensity(kernel="gaussian")
 
     kde, bandwidth, store = kde_util.get_bandwidth(kde, scores_scaler)
 
@@ -95,16 +91,10 @@ def setupKDE(cache):
     h5_data.root.scores_mirror = scores_mirror
     h5_data.root.scores_mirror_scaled = scores_scaler
     h5_data.root.probability = probability
-    h5_data.save()
 
-    cwd = str(Path(__file__).parent)
-    ret = subprocess.run(
-        [sys.executable, "graph_kde.py", str(cache.json_path), str(util.getCoreCounts())],
-        stdout=subprocess.PIPE,
-        stderr=subprocess.PIPE,
-        cwd=cwd,
-    )
-    util.log_subprocess("graph_kde.py", ret)
+    h5_data.save(lock=True)
+
+    sub.graph_kde(cache)
 
     return kde, scaler
 
@@ -175,7 +165,7 @@ def synthetic_error_simulation(x):
     uv_store = {}
     uv_store_norm = {}
 
-    for experiment in cache.cache.settings["kde_synthetic"]:
+    for experiment in cache.cache.settings["errorModel"]:
         delay_settings = experiment["delay"]
         flow_settings = experiment["flow"]
         load_settings = experiment["load"]
@@ -211,15 +201,27 @@ def synthetic_error_simulation(x):
                 comp_name = "solution_outlet_comp_%03d" % comp
 
                 if uv_noise is None:
-                    error_uv = numpy.zeros(len(temp.root.input.solver.user_solution_times))
+                    error_uv = numpy.zeros(
+                        len(temp.root.input.solver.user_solution_times)
+                    )
                 else:
-                    error_uv = numpy.random.normal(uv_noise[0], uv_noise[1], len(temp.root.input.solver.user_solution_times))
+                    error_uv = numpy.random.normal(
+                        uv_noise[0],
+                        uv_noise[1],
+                        len(temp.root.input.solver.user_solution_times),
+                    )
                 uv_store["%s_%s_%s" % (name, unit, comp)] = error_uv
 
                 if uv_noise_norm is None:
-                    error_uv = numpy.ones(len(temp.root.input.solver.user_solution_times))
+                    error_uv = numpy.ones(
+                        len(temp.root.input.solver.user_solution_times)
+                    )
                 else:
-                    error_uv = numpy.random.normal(uv_noise_norm[0], uv_noise_norm[1], len(temp.root.input.solver.user_solution_times))
+                    error_uv = numpy.random.normal(
+                        uv_noise_norm[0],
+                        uv_noise_norm[1],
+                        len(temp.root.input.solver.user_solution_times),
+                    )
                 uv_store_norm["%s_%s_%s" % (name, unit, comp)] = error_uv
 
         def post_function(simulation):
@@ -233,25 +235,27 @@ def synthetic_error_simulation(x):
                     error_uv = uv_store["%s_%s_%s" % (name, unit, comp)]
                     error_norm = uv_store_norm["%s_%s_%s" % (name, unit, comp)]
                     simulation.root.output.solution[unit_name][comp_name] = (
-                        simulation.root.output.solution[unit_name][comp_name] * error_norm + error_uv
+                        simulation.root.output.solution[unit_name][comp_name]
+                        * error_norm
+                        + error_uv
                     )
 
         error_delay = Cadet(temp.root)
 
-        #delays = numpy.random.uniform(delay_settings[0], delay_settings[1], nsec)
-        delays = error[name]['pump_delays']
+        # delays = numpy.random.uniform(delay_settings[0], delay_settings[1], nsec)
+        delays = error[name]["pump_delays"]
         delays_store.extend(delays)
 
         synthetic_error.pump_delay(error_delay, delays)
 
-        #flow = numpy.random.normal(flow_settings[0], flow_settings[1], error_delay.root.input.solver.sections.nsec)
-        flow = error[name]['flow_rates']
+        # flow = numpy.random.normal(flow_settings[0], flow_settings[1], error_delay.root.input.solver.sections.nsec)
+        flow = error[name]["flow_rates"]
         flows_store.extend(flow)
 
         synthetic_error.error_flow(error_delay, flow)
 
-        #load = numpy.random.normal(load_settings[0], load_settings[1], error_delay.root.input.solver.sections.nsec)
-        load = error[name]['loading_concentrations']
+        # load = numpy.random.normal(load_settings[0], load_settings[1], error_delay.root.input.solver.sections.nsec)
+        load = error[name]["loading_concentrations"]
         load_store.extend(load)
 
         synthetic_error.error_load(error_delay, load)
@@ -280,10 +284,16 @@ def synthetic_error_simulation(x):
 
             for unit in units:
                 unit_name = "unit_%03d" % unit
-                for comp in range(result["simulation"].root.input.model[unit_name].ncomp):
-                    outputs["%s_unit_%03d_comp_%03d" % (name, int(unit), comp)] = result["simulation"].root.output.solution[
+                for comp in range(
+                    result["simulation"].root.input.model[unit_name].ncomp
+                ):
+                    outputs[
+                        "%s_unit_%03d_comp_%03d" % (name, int(unit), comp)
+                    ] = result["simulation"].root.output.solution[
                         "unit_%03d" % int(unit)
-                    ]["solution_outlet_comp_%03d" % comp]
+                    ][
+                        "solution_outlet_comp_%03d" % comp
+                    ]
         else:
             experiment_failed = True
 
@@ -298,10 +308,11 @@ def synthetic_error_simulation(x):
 
     return scores, simulations, outputs, errors
 
+
 def get_section_counts(cache):
     sections = []
 
-    for experiment in cache.cache.settings["kde_synthetic"]:
+    for experiment in cache.cache.settings["errorModel"]:
         name = experiment["name"]
 
         if "file_path" in experiment:
@@ -323,14 +334,14 @@ def get_section_counts(cache):
 
     return sections
 
+
 def generate_error_sequence(cache):
     errors = {}
-
-    for experiment in cache.settings["kde_synthetic"]:
+    count = cache.settings["errorModelCount"]
+    for experiment in cache.settings["errorModel"]:
         delay_settings = experiment["delay"]
         flow_settings = experiment["flow"]
         load_settings = experiment["load"]
-        count = experiment['count']
         name = experiment["name"]
 
         if "file_path" in experiment:
@@ -350,31 +361,33 @@ def generate_error_sequence(cache):
 
         error = {}
 
-        #total space is 1 delay per section 1 flow rate per new section and 1 load per new section
+        # total space is 1 delay per section 1 flow rate per new section and 1 load per new section
         delay_size = nsec
-        flow_size = 2*nsec
-        load_size = 2*nsec
+        flow_size = 2 * nsec
+        load_size = 2 * nsec
         total_size = delay_size + flow_size + load_size
 
         errors_exp = SALib.sample.sobol_sequence.sample(count, total_size)
 
-        #set delays
-        block = errors_exp[:,:delay_size]
+        # set delays
+        block = errors_exp[:, :delay_size]
         if delay_settings[0] != delay_settings[1]:
-            block = scipy.stats.uniform.ppf(block, delay_settings[0], delay_settings[1])
+            block = scipy.stats.uniform.ppf(
+                block, delay_settings[0], delay_settings[1] - delay_settings[0]
+            )
         else:
             block = numpy.ones(block.shape) * delay_settings[0]
 
-        error['pump_delays'] = block
+        error["pump_delays"] = block
 
-        #set flow range changes
+        # set flow range changes
         lb = delay_size
         ub = delay_size + flow_size
 
-        #grab the block to work with easier
-        block = errors_exp[:,lb:ub]
+        # grab the block to work with easier
+        block = errors_exp[:, lb:ub]
 
-        #sobol sequence is [0, 1) and 0 needs to be removed and set to the same distance as the closest distance to 1
+        # sobol sequence is [0, 1) and 0 needs to be removed and set to the same distance as the closest distance to 1
         min_value = 1 - numpy.max(block)
 
         block[block < min_value] = min_value
@@ -384,36 +397,36 @@ def generate_error_sequence(cache):
         else:
             block = numpy.ones(block.shape) * flow_settings[0]
 
-        error['flow_rates'] = block
+        error["flow_rates"] = block
 
-
-        #set loading changes
+        # set loading changes
         lb = ub
         ub = lb + load_size
 
-        #grab the block to work with easier
-        block = errors_exp[:,lb:ub]
+        # grab the block to work with easier
+        block = errors_exp[:, lb:ub]
 
-        #sobol sequence is [0, 1) and 0 needs to be removed and set to the same distance as the closest distance to 1
+        # sobol sequence is [0, 1) and 0 needs to be removed and set to the same distance as the closest distance to 1
         min_value = 1 - numpy.max(block)
 
-        block[block < min_value] = min_value        
+        block[block < min_value] = min_value
 
         if load_settings[1] != 0:
             block = scipy.stats.norm.ppf(block, load_settings[0], load_settings[1])
         else:
             block = numpy.ones(block.shape) * load_settings[0]
 
-        error['loading_concentrations'] = block
+        error["loading_concentrations"] = block
 
         errors[name] = error
 
     return errors
 
+
 def split_errors(errors_all):
     errors = []
-    for name,experiment in errors_all.items():
-        length = len(experiment['flow_rates'])
+    for name, experiment in errors_all.items():
+        length = len(experiment["flow_rates"])
         break
 
     temp = {}
@@ -421,32 +434,33 @@ def split_errors(errors_all):
         temp[key] = {}
 
     for i in range(length):
-        errors.append({key:{} for key in errors_all})
-    
-    for name,experiment in errors_all.items():
+        errors.append({key: {} for key in errors_all})
+
+    for name, experiment in errors_all.items():
         for error_name, error_value in experiment.items():
             for idx in range(length):
-                errors[idx][name][error_name] = error_value[idx,:]
+                errors[idx][name][error_name] = error_value[idx, :]
 
     return errors
 
+
 def keep_data(data, lb=0.5, ub=99.5):
     cutoffs_lb, cutoffs_ub = numpy.percentile(data, [lb, ub], axis=0)
     remove = numpy.zeros(data.shape)
-    
-    for i in range(data.shape[1]-2):
-        remove[:,i] = data[:,i] > cutoffs_lb[i]
-    remove[:,-2] = (data[:,-2] > cutoffs_lb[-2]) & (data[:,-2] < cutoffs_ub[-2])
-    remove[:,-1] = (data[:,-1] > cutoffs_lb[-1]) & (data[:,-1] < cutoffs_ub[-1])
-    
+
+    for i in range(data.shape[1] - 2):
+        remove[:, i] = data[:, i] > cutoffs_lb[i]
+    remove[:, -2] = (data[:, -2] > cutoffs_lb[-2]) & (data[:, -2] < cutoffs_ub[-2])
+    remove[:, -1] = (data[:, -1] > cutoffs_lb[-1]) & (data[:, -1] < cutoffs_ub[-1])
+
     remove = numpy.all(remove, axis=1)
-    
+
     return remove
 
-def generate_synthetic_error(cache):
-    if "kde_synthetic" in cache.settings:
-        count_settings = int(cache.settings["kde_synthetic"][0]["count"])
 
+def generate_synthetic_error(cache):
+    count_settings = int(cache.settings["errorModelCount"])
+    if "errorModel" in cache.settings:
         scores_all = []
         times = {}
         outputs_all = {}
@@ -460,7 +474,9 @@ def generate_synthetic_error(cache):
 
         indexes = []
 
-        for idx, (scores, simulations, outputs, errors) in enumerate(cache.toolbox.map(synthetic_error_simulation, zip(json_path, errors_split))):
+        for idx, (scores, simulations, outputs, errors) in enumerate(
+            cache.toolbox.map(synthetic_error_simulation, zip(json_path, errors_split))
+        ):
             if scores and simulations and outputs:
                 indexes.append(idx)
 
@@ -482,9 +498,9 @@ def generate_synthetic_error(cache):
 
         indexes = numpy.array(indexes)
 
-        for name,experiment in errors_all.items():
+        for name, experiment in errors_all.items():
             for error_name, error_value in experiment.items():
-                errors_all[name][error_name] = error_value[indexes,:]
+                errors_all[name][error_name] = error_value[indexes, :]
 
         scores = numpy.array(scores_all)
 
@@ -507,25 +523,27 @@ def generate_synthetic_error(cache):
         kde_data.root.original.scores = scores
 
         for output_name, output in outputs_all.items():
-            kde_data.root[output_name] = numpy.array(output)[keep_idx,:]
-            kde_data.root.original[output_name] = numpy.array(output)[keep_idx,:]
+            kde_data.root[output_name] = numpy.array(output)[keep_idx, :]
+            kde_data.root.original[output_name] = numpy.array(output)[keep_idx, :]
 
         for time_name, time in times.items():
             kde_data.root["%s_time" % time_name] = time
             kde_data.root.original["%s_time" % time_name] = time
 
-        for name,experiment in errors_all.items():
+        for name, experiment in errors_all.items():
             for error_name, error_value in experiment.items():
                 kde_data.root.errors[name][error_name] = error_value[keep_idx, :]
-                kde_data.root.original.errors[name][error_name] = error_value[keep_idx, :]
+                kde_data.root.original.errors[name][error_name] = error_value[
+                    keep_idx, :
+                ]
 
         for key, value in uv_store_all.items():
-            kde_data.root.uv_store[key] = numpy.array(value)[keep_idx,:]
-            kde_data.root.original.uv_store[key] = numpy.array(value)[keep_idx,:]
+            kde_data.root.uv_store[key] = numpy.array(value)[keep_idx, :]
+            kde_data.root.original.uv_store[key] = numpy.array(value)[keep_idx, :]
 
-        kde_data.save()
+        kde_data.save(lock=True)
 
-        return scores[keep_idx,:]
+        return scores[keep_idx, :]
 
     return None
 
@@ -533,4 +551,9 @@ def generate_synthetic_error(cache):
 def convert_to_array(seq):
     "this converts a sequence of arrays to a single numpy array and pads uneven rows with 0"
     max_len = numpy.max([len(a) for a in seq])
-    return numpy.asarray([numpy.pad(a, (0, max_len - len(a)), "constant", constant_values=0) for a in seq])
+    return numpy.asarray(
+        [
+            numpy.pad(a, (0, max_len - len(a)), "constant", constant_values=0)
+            for a in seq
+        ]
+    )
