@@ -1,7 +1,8 @@
-import CADETMatch.util as util
+from addict import Dict
+
 import CADETMatch.calc_coeff as calc_coeff
 import CADETMatch.score as score
-from addict import Dict
+import CADETMatch.util as util
 
 name = "Ceiling"
 
@@ -12,6 +13,9 @@ def get_settings(feature):
     settings.badScore = 0
     settings.meta_mask = True
     settings.count = 1
+    settings.graph_der = 0
+    settings.graph = 1
+    settings.graph_frac = 0
     return settings
 
 
@@ -19,7 +23,9 @@ def run(sim_data, feature):
     "similarity, value, start stop"
     selected = feature["selected"]
     exp_data_values = feature["value"][selected]
-    sim_time_values, sim_data_values = util.get_times_values(sim_data["simulation"], feature)
+    sim_time_values, sim_data_values = util.get_times_values(
+        sim_data["simulation"], feature
+    )
 
     max_value = max(sim_data_values)
 
@@ -45,6 +51,7 @@ def run(sim_data, feature):
 
 def setup(sim, feature, selectedTimes, selectedValues, CV_time, abstol, cache):
     temp = {}
+    temp["peak_max"] = max(selectedValues)
     temp["value_function"] = score.value_function(feature["max_value"], abstol)
     return temp
 
