@@ -1,8 +1,10 @@
-import CADETMatch.util as util
-import CADETMatch.score as score
+import numpy
 import scipy.stats
 from addict import Dict
-import numpy
+
+import CADETMatch.score as score
+import CADETMatch.util as util
+import CADETMatch.smoothing as smoothing
 
 name = "similarity"
 
@@ -13,12 +15,17 @@ def get_settings(feature):
     settings.badScore = 0
     settings.meta_mask = True
     settings.count = 3
+    settings.graph_der = 0
+    settings.graph = 1
+    settings.graph_frac = 0
     return settings
 
 
 def run(sim_data, feature):
     "similarity, value, start stop"
-    sim_time_values, sim_data_values = util.get_times_values(sim_data["simulation"], feature)
+    sim_time_values, sim_data_values = util.get_times_values(
+        sim_data["simulation"], feature
+    )
     selected = feature["selected"]
 
     exp_data_values = feature["value"][selected]
@@ -28,9 +35,15 @@ def run(sim_data, feature):
 
     time_high, value_high = high
 
-    pearson, diff_time = score.pearson_spline(exp_time_values, sim_data_values, feature["smooth_value"])
+    pearson, diff_time = score.pearson_spline(
+        exp_time_values, sim_data_values, feature["smooth_value"]
+    )
 
-    temp = [pearson, feature["value_function"](value_high), feature["time_function"](numpy.abs(diff_time))]
+    temp = [
+        pearson,
+        feature["value_function"](value_high),
+        feature["time_function"](numpy.abs(diff_time)),
+    ]
     return (
         temp,
         util.sse(sim_data_values, exp_data_values),
@@ -44,7 +57,9 @@ def run(sim_data, feature):
 
 def setup(sim, feature, selectedTimes, selectedValues, CV_time, abstol, cache):
     name = "%s_%s" % (sim.root.experiment_name, feature["name"])
-    s, crit_fs, crit_fs_der = smoothing.find_smoothing_factors(selectedTimes, selectedValues, name, cache)
+    s, crit_fs, crit_fs_der = smoothing.find_smoothing_factors(
+        selectedTimes, selectedValues, name, cache
+    )
 
     temp = {}
     temp["peak"] = util.find_peak(selectedTimes, selectedValues)[0]
@@ -55,7 +70,9 @@ def setup(sim, feature, selectedTimes, selectedValues, CV_time, abstol, cache):
     temp["smoothing_factor"] = s
     temp["critical_frequency"] = crit_fs
     temp["critical_frequency_der"] = crit_fs_der
-    temp["smooth_value"] = smoothing.smooth_data(selectedTimes, selectedValues, crit_fs, s)
+    temp["smooth_value"] = smoothing.smooth_data(
+        selectedTimes, selectedValues, crit_fs, s
+    )
     return temp
 
 
