@@ -1,7 +1,9 @@
-import CADETMatch.util as util
 import sys
-from addict import Dict
+
 import numpy
+from addict import Dict
+
+import CADETMatch.util as util
 
 name = "SSE"
 
@@ -12,12 +14,17 @@ def get_settings(feature):
     settings.badScore = -sys.float_info.max
     settings.meta_mask = True
     settings.count = 1
+    settings.graph_der = 0
+    settings.graph = 1
+    settings.graph_frac = 0
     return settings
 
 
 def run(sim_data, feature):
     "sum square error score, this score is NOT composable with other scores, use negative so score is maximized like other scores"
-    sim_time_values, sim_data_values = util.get_times_values(sim_data["simulation"], feature)
+    sim_time_values, sim_data_values = util.get_times_values(
+        sim_data["simulation"], feature
+    )
     selected = feature["selected"]
 
     exp_time_values = feature["time"][selected]
@@ -25,7 +32,19 @@ def run(sim_data, feature):
 
     sse = util.sse(sim_data_values, exp_data_values)
 
-    return ([-sse,], sse, len(sim_data_values), sim_time_values, sim_data_values, exp_data_values, [sse,])
+    return (
+        [
+            -sse,
+        ],
+        sse,
+        len(sim_data_values),
+        sim_time_values,
+        sim_data_values,
+        exp_data_values,
+        [
+            sse,
+        ],
+    )
 
 
 def setup(sim, feature, selectedTimes, selectedValues, CV_time, abstol, cache):
