@@ -1,5 +1,6 @@
-import CADETMatch.util as util
 import numpy
+
+import CADETMatch.util as util
 from CADETMatch.abstract.transform import AbstractTransform
 
 
@@ -27,9 +28,16 @@ class VolumeLengthTransform(AbstractTransform):
 
     grad_transform = transform
 
+    def untransform_inputorder(self, seq):
+        volume = seq[0]
+        length = seq[1]
+        return [volume, length]
+
     def untransform(self, seq):
-        values = [seq[0] / seq[1], seq[1]]
-        headerValues = [values[0], values[1], values[0] * values[1]]
+        volume, length = self.untransform_inputorder(seq)
+        area = volume / length
+        values = [area, length]
+        headerValues = [area, length, volume]
         return values, headerValues
 
     def grad_untransform(self, seq):
@@ -49,7 +57,10 @@ class VolumeLengthTransform(AbstractTransform):
     def setSimulation(self, sim, seq, experiment):
         values, headerValues = self.untransform(seq)
 
-        if self.parameter.get("experiments", None) is None or experiment["name"] in self.parameter["experiments"]:
+        if (
+            self.parameter.get("experiments", None) is None
+            or experiment["name"] in self.parameter["experiments"]
+        ):
             area_location = self.parameter["area_location"]
             length_location = self.parameter["length_location"]
 
@@ -86,8 +97,8 @@ class VolumeLengthTransform(AbstractTransform):
         return headers
 
     def setBounds(self, parameter, lb, ub):
-        parameter["minVolume"] = lb[2]
-        parameter["maxVolume"] = ub[2]
+        parameter["minVolume"] = lb[0]
+        parameter["maxVolume"] = ub[0]
         parameter["minLength"] = lb[1]
         parameter["maxLength"] = ub[1]
 
@@ -113,7 +124,7 @@ class NormVolumeLengthTransform(VolumeLengthTransform):
 
     grad_transform = transform
 
-    def untransform(self, seq):
+    def untransform_inputorder(self, seq):
         minVolume = self.parameter["minVolume"]
         maxVolume = self.parameter["maxVolume"]
         minLength = self.parameter["minLength"]
@@ -125,10 +136,7 @@ class NormVolumeLengthTransform(VolumeLengthTransform):
         values = numpy.array(seq)
 
         values = (maxValues - minValues) * values + minValues
-
-        values = [values[0] / values[1], values[1]]
-        headerValues = [values[0], values[1], values[0] * values[1]]
-        return values, headerValues
+        return values
 
     def grad_untransform(self, seq):
         return self.untransform(seq)[0]
@@ -155,7 +163,12 @@ class NormVolumeLengthTransform(VolumeLengthTransform):
         return [0.0, 0.0], [1.0, 1.0]
 
     def getGradBounds(self):
-        return [self.parameter["min"],], [self.parameter["max"],]
+        return [self.parameter["min"],], [
+            self.parameter["max"],
+        ]
 
 
-plugins = {"norm_volume_length": NormVolumeLengthTransform, "volume_length": VolumeLengthTransform}
+plugins = {
+    "norm_volume_length": NormVolumeLengthTransform,
+    "volume_length": VolumeLengthTransform,
+}
