@@ -1,51 +1,44 @@
-import numpy
-import pandas
-
+import copy
+import csv
+import decimal as decim
 import hashlib
-
-from deap import tools
-import scipy.signal
-from pathlib import Path
-
-from addict import Dict
-
-import tempfile
+import multiprocessing
 import os
-from cadet import Cadet, H5
+import random
 import subprocess
 import sys
-import jstyleson
+import tempfile
 import time
-import csv
-import psutil
-
-import random
-import CADETMatch.calc_coeff as calc_coeff
 import warnings
+from pathlib import Path
 
-with warnings.catch_warnings():
-    warnings.filterwarnings("ignore", category=FutureWarning)
-    import h5py
+import jstyleson
+import numpy
+import pandas
+import psutil
+import scipy.signal
+from addict import Dict
+from cadet import H5, Cadet
+from deap import tools
 
-import multiprocessing
-import copy
-
-import decimal as decim
+import CADETMatch.calc_coeff as calc_coeff
+import CADETMatch.sub as sub
 
 decim.getcontext().prec = 64
 __logBase10of2_decim = decim.Decimal(2).log10()
 __logBase10of2 = float(__logBase10of2_decim)
 
-import SALib.sample.sobol_sequence
-import CADETMatch.loggerwriter as loggerwriter
-import CADETMatch.synthetic_error as synthetic_error
-import CADETMatch.pareto as pareto
-
 import logging
 import os
+
+import SALib.sample.sobol_sequence
 import scipy.interpolate
 
-#numpy.warnings.filterwarnings('error', category=numpy.VisibleDeprecationWarning)
+import CADETMatch.loggerwriter as loggerwriter
+import CADETMatch.pareto as pareto
+import CADETMatch.synthetic_error as synthetic_error
+
+# numpy.warnings.filterwarnings('error', category=numpy.VisibleDeprecationWarning)
 
 
 def get_times_values(simulation, target, selected=None):
@@ -68,6 +61,7 @@ def get_times_values(simulation, target, selected=None):
 
     return times[selected], values[selected] * target["factor"]
 
+
 def sse(data1, data2):
     return numpy.sum((data1 - data2) ** 2)
 
@@ -147,10 +141,14 @@ def saveExperiments(save_name_base, settings, target, results, directory, file_p
 
         same = False
         if dst.is_file():
-            #load tolerance to see if the files are actually the same
+            # load tolerance to see if the files are actually the same
             sim_dst = Cadet()
             sim_dst.filename = dst.as_posix()
-            sim_dst.load(paths=['/input/solver/time_integrator',])
+            sim_dst.load(
+                paths=[
+                    "/input/solver/time_integrator",
+                ]
+            )
 
             dst_tol = sim_dst.root.input.solver.time_integrator.abstol
             sim_tol = simulation.root.input.solver.time_integrator.abstol
@@ -161,13 +159,30 @@ def saveExperiments(save_name_base, settings, target, results, directory, file_p
         else:
             simulation.filename = dst.as_posix()
 
-            for (header, score) in zip(experiment["headers"], results[experimentName]["scores"]):
+            for (header, score) in zip(
+                experiment["headers"], results[experimentName]["scores"]
+            ):
                 simulation.root.score[header] = score
             simulation.save()
 
     return True
 
 
+def convert_individual_inputorder(individual, cache):
+    cadetValues = []
+
+    idx = 0
+    for parameter in cache.parameters:
+        count = parameter.count
+        if count:
+            seq = individual[idx : idx + count]
+            values = parameter.untransform_inputorder(seq)
+            cadetValues.extend(values)
+            idx += count
+
+    return cadetValues
+
+
 def convert_individual(individual, cache):
     cadetValues = []
     cadetValuesExtended = []
@@ -216,18 +231,21 @@ def convert_population(population, cache):
 
 
 def convert_population_inputorder(population, cache):
-    cadetValues = numpy.zeros(population.shape)
+    if len(population):
+        cadetValues = numpy.zeros(population.shape)
 
-    idx = 0
-    for parameter in cache.parameters:
-        count = parameter.count
-        if count:
-            matrix = population[:, idx : idx + count]
-            values = parameter.untransform_matrix_inputorder(matrix)
-            cadetValues[:, idx : idx + count] = values
-            idx += count
+        idx = 0
+        for parameter in cache.parameters:
+            count = parameter.count
+            if count:
+                matrix = population[:, idx : idx + count]
+                values = parameter.untransform_matrix_inputorder(matrix)
+                cadetValues[:, idx : idx + count] = values
+                idx += count
 
-    return cadetValues
+        return cadetValues
+    else:
+        return numpy.array([])
 
 
 def convert_individual_inverse(individual, cache):
@@ -235,7 +253,9 @@ def convert_individual_inverse(individual, cache):
 
 
 def convert_individual_inverse_grad(individual, cache):
-    return numpy.array([f(v) for f, v in zip(cache.settings["grad_transform"], individual)])
+    return numpy.array(
+        [f(v) for f, v in zip(cache.settings["grad_transform"], individual)]
+    )
 
 
 def set_simulation(individual, simulation, settings, cache, experiment):
@@ -266,13 +286,31 @@ def getBoundOffset(unit):
 
     if not len(NBOUND):
         "If NBOUND is empty it is all zero"
-        NBOUND = [0.0] * unit.ncomp
+        NBOUND = [0] * unit.ncomp
 
-    boundOffset = numpy.cumsum(numpy.concatenate([[0,], NBOUND]))
+    boundOffset = numpy.cumsum(
+        numpy.concatenate(
+            [
+                [
+                    0,
+                ],
+                NBOUND,
+            ]
+        )
+    )
     return boundOffset
 
 
-def runExperiment(individual, experiment, settings, target, template_sim, timeout, cache, post_function=None):
+def runExperiment(
+    individual,
+    experiment,
+    settings,
+    target,
+    template_sim,
+    timeout,
+    cache,
+    post_function=None,
+):
     handle, path = tempfile.mkstemp(suffix=".h5", dir=cache.tempDir)
     os.close(handle)
 
@@ -282,7 +320,9 @@ def runExperiment(individual, experiment, settings, target, template_sim, timeou
     simulation.root.input.solver.nthreads = int(settings.get("nThreads", 1))
 
     if individual is not None:
-        cadetValues, cadetValuesKEQ = set_simulation(individual, simulation, settings, cache, experiment)
+        cadetValues, cadetValuesKEQ = set_simulation(
+            individual, simulation, settings, cache, experiment
+        )
     else:
         cadetValues = []
         cadetValuesKEQ = []
@@ -307,7 +347,9 @@ def runExperiment(individual, experiment, settings, target, template_sim, timeou
 
     simulationFailed = isinstance(simulation.root.output.solution.solution_times, Dict)
     if simulationFailed:
-        multiprocessing.get_logger().error("%s sim must have failed %s", individual, path)
+        multiprocessing.get_logger().error(
+            "%s sim must have failed %s", individual, path
+        )
         return None
     multiprocessing.get_logger().debug("Everything ran fine")
 
@@ -337,7 +379,15 @@ def runExperiment(individual, experiment, settings, target, template_sim, timeou
 
         if featureType in cache.scores:
             try:
-                scores, sse, sse_count, sim_time, sim_value, exp_value, minimize = cache.scores[featureType].run(
+                (
+                    scores,
+                    sse,
+                    sse_count,
+                    sim_time,
+                    sim_value,
+                    exp_value,
+                    minimize,
+                ) = cache.scores[featureType].run(
                     temp, target[experiment["name"]][featureName]
                 )
             except TypeError:
@@ -434,14 +484,18 @@ def setupMCMC(cache):
                 if feature["type"] == "AbsoluteHeight":
                     foundAbsoluteHeight = True
             if foundAbsoluteTime is False:
-                experiment["features"].append({"name": "AbsoluteTime", "type": "AbsoluteTime"})
+                experiment["features"].append(
+                    {"name": "AbsoluteTime", "type": "AbsoluteTime"}
+                )
             if foundAbsoluteHeight is False:
-                experiment["features"].append({"name": "AbsoluteHeight", "type": "AbsoluteHeight"})
+                experiment["features"].append(
+                    {"name": "AbsoluteHeight", "type": "AbsoluteHeight"}
+                )
 
-        if "kde_synthetic" in settings:
+        if "errorModel" in settings:
             individual = getBestIndividual(cache)
             found = createSimulationBestIndividual(individual, cache)
-            for experiment in settings["kde_synthetic"]:
+            for experiment in settings["errorModel"]:
                 experiment["file_path"] = found[experiment["name"]]
 
         new_settings_file = resultDir / settings_file.name
@@ -453,25 +507,30 @@ def setupMCMC(cache):
 def update_json_mcmc(settings):
     data = H5()
     data.filename = settings["mcmc_h5"]
-    data.load(paths=["/bounds_change/json"])
-    json_data = jstyleson.loads(str(data.root.bounds_change.json, 'ascii'))
+    data.load(paths=["/bounds_change/json"], lock=True)
+    prior_parameters = jstyleson.loads(str(data.root.bounds_change.json, "ascii"))
+
+    keep = ['transform', 'min', 'max', 'minKA', 'maxKA',
+                      'minKEQ', 'maxKEQ', 'factor', 'minNu', 'maxNu',
+                      'minSigma', 'maxSigma', 'minVolume', 'maxVolume',
+                      'minArea', 'maxArea', 'minLength', 'maxLength']
 
     if "parameters_mcmc" in settings:
         new_parameters = settings["parameters_mcmc"]
 
-        for a, b in zip(new_parameters, json_data):
-            if a["transform"] == b["transform"] and a["location"].split("/")[-1] == b["location"].split("/")[-1]:
-                a["min"] = b["min"]
-                a["max"] = b["max"]
+        for new, prior in zip(new_parameters, prior_parameters):
+            if new["location"].split("/")[-1] == prior["location"].split("/")[-1]:
+                #update just the location data everthing else needs to remain the same
+                for key, value in new.items():
+                    if key not in keep:
+                        prior[key] = new[key]
             else:
                 multiprocessing.get_logger().info(
                     "parameters_mcmc does not have the same transform and variables in the same order as the prior, MCMC cannot continue until this is fixed"
                 )
                 sys.exit()
-
-        settings["parameters"].extend(new_parameters)
-    else:
-        settings["parameters"].extend(json_data)
+       
+    settings["parameters"].extend(prior_parameters)
 
 
 def setupAltFeature(cache, name):
@@ -515,12 +574,16 @@ def createSimulationBestIndividual(individual, cache):
     temp = {}
     for experiment in cache.settings["experiments"]:
         name = experiment["name"]
-        templatePath = Path(cache.settings["resultsDirMisc"], "template_%s_base.h5" % name)
+        templatePath = Path(
+            cache.settings["resultsDirMisc"], "template_%s_base.h5" % name
+        )
         templateSim = Cadet()
         templateSim.filename = templatePath.as_posix()
         templateSim.load()
 
-        cadetValues, cadetValuesKEQ = set_simulation(individual, templateSim, cache.settings, cache, experiment)
+        cadetValues, cadetValuesKEQ = set_simulation(
+            individual, templateSim, cache.settings, cache, experiment
+        )
 
         bestPath = Path(cache.settings["resultsDirMisc"], "best_%s_base.h5" % name)
         templateSim.filename = bestPath.as_posix()
@@ -534,7 +597,7 @@ def getBestIndividual(cache):
     progress_path = Path(cache.settings["resultsDirBase"]) / "result.h5"
     results = H5()
     results.filename = progress_path.as_posix()
-    results.load(paths=["/meta_score", "/meta_population"])
+    results.load(paths=["/meta_score", "/meta_population"], lock=True)
 
     idx = numpy.argmax(results.root.meta_score[:, 1])
     individual = results.root.meta_population[idx, :]
@@ -597,8 +660,12 @@ def fracStat(time_center, value):
     skew_time = numpy.sum((time_center - mean_time) ** 3 * value) / numpy.sum(value)
 
     mean_value = numpy.sum(time_center * value) / numpy.sum(time_center)
-    variance_value = numpy.sum((value - mean_value) ** 2 * time_center) / numpy.sum(time_center)
-    skew_value = numpy.sum((value - mean_value) ** 3 * time_center) / numpy.sum(time_center)
+    variance_value = numpy.sum((value - mean_value) ** 2 * time_center) / numpy.sum(
+        time_center
+    )
+    skew_value = numpy.sum((value - mean_value) ** 3 * time_center) / numpy.sum(
+        time_center
+    )
 
     return mean_time, variance_time, skew_time, mean_value, variance_value, skew_value
 
@@ -610,26 +677,34 @@ def fractionate_spline(start_seq, stop_seq, spline):
     return numpy.array(temp)
 
 
-def find_opt_poly(x,y, index):
+def find_opt_poly(x, y, index):
     if index == 0:
-        indexes = numpy.array([index, index+1, index+2])
-    elif index == (len(x)-1):
-        indexes = numpy.array([index-2, index-1, index])
+        indexes = numpy.array([index, index + 1, index + 2])
+    elif index == (len(x) - 1):
+        indexes = numpy.array([index - 2, index - 1, index])
     else:
-        indexes = numpy.array([index-1, index, index+1])
+        indexes = numpy.array([index - 1, index, index + 1])
 
     x = x[indexes]
     y = y[indexes]
 
-    if x[0] > x[-1]:  #need to invert order
+    if x[0] > x[-1]:  # need to invert order
         x = x[::-1]
         y = y[::-1]
 
-    poly, res = numpy.polynomial.Polynomial.fit(x, y,2, full=True)
-    root = poly.deriv().roots()[0]
+    try:
+        poly, res = numpy.polynomial.Polynomial.fit(x, y, 2, full=True)
+    except numpy.linalg.LinAlgError:
+        multiprocessing.get_logger().error("Polyfit failed for  %s %s", x, y)
+    try:
+        root = poly.deriv().roots()[0]
+    except IndexError:
+        # this happens if all y values are the same in which case just take the center value
+        root = x[0]
     root = numpy.clip(root, x[0], x[1])
     return root, x, y
 
+
 def fractionate(start_seq, stop_seq, times, values):
     temp = []
     for (start, stop) in zip(start_seq, stop_seq):
@@ -776,7 +851,9 @@ def update_result_data(cache, ind, fit, result_data, results, meta_scores):
 
         for result in results.values():
             result_data["input_transform"].append(tuple(result["cadetValues"]))
-            result_data["input_transform_extended"].append(tuple(result["cadetValuesKEQ"]))
+            result_data["input_transform_extended"].append(
+                tuple(result["cadetValuesKEQ"])
+            )
 
             # All results have the same parameter set so we only need the first one
             break
@@ -821,7 +898,17 @@ def calcFitness(scores_orig, cache):
 
 
 def process_population(
-    toolbox, cache, population, fitnesses, writer, csvfile, halloffame, meta_hof, progress_hof, generation, result_data=None
+    toolbox,
+    cache,
+    population,
+    fitnesses,
+    writer,
+    csvfile,
+    halloffame,
+    meta_hof,
+    progress_hof,
+    generation,
+    result_data=None,
 ):
     csv_lines = []
     meta_csv_lines = []
@@ -844,7 +931,11 @@ def process_population(
         if (time.time() - last_time) > elapsed:
             percent = idx / len(population)
             multiprocessing.get_logger().info(
-                "Generation %s approximately %.1f %% complete with %s/%s done", generation, percent * 100, idx, len(population)
+                "Generation %s approximately %.1f %% complete with %s/%s done",
+                generation,
+                percent * 100,
+                idx,
+                len(population),
             )
             last_time = time.time()
 
@@ -852,7 +943,9 @@ def process_population(
 
         ind = pop_lookup(lookup, individual)
 
-        save_name_base = hashlib.md5(str(list(ind)).encode("utf-8", "ignore")).hexdigest()
+        save_name_base = hashlib.md5(
+            str(list(ind)).encode("utf-8", "ignore")
+        ).hexdigest()
 
         ind.fitness.values = calcFitness(fit, cache)
         ind.csv_line = [time.ctime(), save_name_base] + csv_line
@@ -872,7 +965,9 @@ def process_population(
                 if onFront and not cache.metaResultsOnly:
                     processResults(save_name_base, ind, cache, results)
 
-            onFrontMeta, significant_meta = pareto.updateParetoFront(meta_hof, ind_meta, cache)
+            onFrontMeta, significant_meta = pareto.updateParetoFront(
+                meta_hof, ind_meta, cache
+            )
             if onFrontMeta:
                 meta_csv_lines.append([time.ctime(), save_name_base] + csv_line)
                 processResultsMeta(save_name_base, ind, cache, results)
@@ -880,7 +975,9 @@ def process_population(
             # If this is None progress can never be made and the algorithm will terminate, it should only be
             # None if used with algorithms like multistart or scoretest which don't need progress
             if progress_hof is not None:
-                onFrontProgress, significant_progress = pareto.updateParetoFront(copy.deepcopy(progress_hof), ind_meta, cache)
+                onFrontProgress, significant_progress = pareto.updateParetoFront(
+                    copy.deepcopy(progress_hof), ind_meta, cache
+                )
                 if significant_progress:
                     made_progress = True
                     pareto.updateParetoFront(progress_hof, ind_meta, cache)
@@ -922,40 +1019,103 @@ def process_population(
     return stalled, stallWarn, progressWarn
 
 
-def eval_population(toolbox, cache, invalid_ind, writer, csvfile, halloffame, meta_hof, progress_hof, generation, result_data=None):
+def eval_population(
+    toolbox,
+    cache,
+    invalid_ind,
+    writer,
+    csvfile,
+    halloffame,
+    meta_hof,
+    progress_hof,
+    generation,
+    result_data=None,
+):
     return eval_population_base(
-        toolbox.evaluate, toolbox, cache, invalid_ind, writer, csvfile, halloffame, meta_hof, progress_hof, generation, result_data
+        toolbox.evaluate,
+        toolbox,
+        cache,
+        invalid_ind,
+        writer,
+        csvfile,
+        halloffame,
+        meta_hof,
+        progress_hof,
+        generation,
+        result_data,
     )
 
 
-def eval_population_final(toolbox, cache, invalid_ind, writer, csvfile, halloffame, meta_hof, progress_hof, generation, result_data=None):
+def eval_population_final(
+    toolbox,
+    cache,
+    invalid_ind,
+    writer,
+    csvfile,
+    halloffame,
+    meta_hof,
+    progress_hof,
+    generation,
+    result_data=None,
+):
     return eval_population_base(
-        toolbox.evaluate_final, toolbox, cache, invalid_ind, writer, csvfile, halloffame, meta_hof, progress_hof, generation, result_data
+        toolbox.evaluate_final,
+        toolbox,
+        cache,
+        invalid_ind,
+        writer,
+        csvfile,
+        halloffame,
+        meta_hof,
+        progress_hof,
+        generation,
+        result_data,
     )
 
 
 def eval_population_base(
-    evaluate, toolbox, cache, invalid_ind, writer, csvfile, halloffame, meta_hof, progress_hof, generation, result_data=None
+    evaluate,
+    toolbox,
+    cache,
+    invalid_ind,
+    writer,
+    csvfile,
+    halloffame,
+    meta_hof,
+    progress_hof,
+    generation,
+    result_data=None,
 ):
     fitnesses = toolbox.map(evaluate, map(list, invalid_ind))
 
     return process_population(
-        toolbox, cache, invalid_ind, fitnesses, writer, csvfile, halloffame, meta_hof, progress_hof, generation, result_data
+        toolbox,
+        cache,
+        invalid_ind,
+        fitnesses,
+        writer,
+        csvfile,
+        halloffame,
+        meta_hof,
+        progress_hof,
+        generation,
+        result_data,
     )
 
 
 def get_grad_tolerance(cache, name):
-    smallest_peak = cache.target[name]['smallest_peak']
-    largest_peak = cache.target[name]['largest_peak']
+    smallest_peak = cache.target[name]["smallest_peak"]
+    largest_peak = cache.target[name]["largest_peak"]
 
     grad_tol = cache.abstolFactorGrad * smallest_peak
     grad_tol_min = cache.abstolFactorGradMax * largest_peak
 
     return max([grad_tol, grad_tol_min])
 
+
 def get_evo_tolerance(cache, name):
-    smallest_peak = cache.target[name]['smallest_peak']
-    largest_peak = cache.target[name]['largest_peak']
+    smallest_peak = cache.target[name]["smallest_peak"]
+    largest_peak = cache.target[name]["largest_peak"]
 
     evo_tol = cache.abstolFactor * smallest_peak
     evo_tol_min = cache.abstolFactorGradMax * largest_peak
@@ -972,15 +1132,36 @@ def writeMetaFront(cache, meta_hof, path_meta_csv):
 
 
 def processResultsGrad(save_name_base, individual, cache, results):
-    saveExperiments(save_name_base, cache.settings, cache.target, results, cache.settings["resultsDirGrad"], "%s_%s_GRAD.h5")
+    saveExperiments(
+        save_name_base,
+        cache.settings,
+        cache.target,
+        results,
+        cache.settings["resultsDirGrad"],
+        "%s_%s_GRAD.h5",
+    )
 
 
 def processResults(save_name_base, individual, cache, results):
-    saveExperiments(save_name_base, cache.settings, cache.target, results, cache.settings["resultsDirEvo"], "%s_%s_EVO.h5")
+    saveExperiments(
+        save_name_base,
+        cache.settings,
+        cache.target,
+        results,
+        cache.settings["resultsDirEvo"],
+        "%s_%s_EVO.h5",
+    )
 
 
 def processResultsMeta(save_name_base, individual, cache, results):
-    saveExperiments(save_name_base, cache.settings, cache.target, results, cache.settings["resultsDirMeta"], "%s_%s_meta.h5")
+    saveExperiments(
+        save_name_base,
+        cache.settings,
+        cache.target,
+        results,
+        cache.settings["resultsDirMeta"],
+        "%s_%s_meta.h5",
+    )
 
 
 def cleanupFront(cache, halloffame=None, meta_hof=None, grad_hof=None):
@@ -1002,7 +1183,10 @@ def cleanDir(dir, hof):
     exists = {str(path.name).split("_", 1)[0] for path in paths}
 
     # make set of allowed keys based on hall of hame
-    allowed = {hashlib.md5(str(list(individual)).encode("utf-8", "ignore")).hexdigest() for individual in hof.items}
+    allowed = {
+        hashlib.md5(str(list(individual)).encode("utf-8", "ignore")).hexdigest()
+        for individual in hof.items
+    }
 
     # remove everything not in hall of fame
     remove = exists - allowed
@@ -1012,85 +1196,9 @@ def cleanDir(dir, hof):
             path.unlink()
 
 
-def graph_process(cache, generation, last=0):
-    if "lastGraphTime" not in graph_process.__dict__:
-        graph_process.lastGraphTime = time.time()
-    if "lastMetaTime" not in graph_process.__dict__:
-        graph_process.lastMetaTime = time.time()
-
-    if "child" in graph_process.__dict__:
-        if graph_process.child.poll() is None:  # This is false if the child has completed
-            if last:
-                graph_process.child.wait()
-            else:
-                return
-
-    cwd = str(Path(__file__).parent)
-
-    if cache.graphSpearman:  # This is mostly just for debugging now and is not run async
-        ret = subprocess.run(
-            [sys.executable, "graph_spearman.py", str(cache.json_path), str(generation), str(getCoreCounts())],
-            stdout=subprocess.PIPE,
-            stderr=subprocess.PIPE,
-            cwd=cwd,
-        )
-        log_subprocess("graph_spearman.py", ret)
-
-    if last:
-        ret = subprocess.run(
-            [sys.executable, "generate_graphs.py", str(cache.json_path), str(cache.graphType), str(getCoreCounts())],
-            stdout=subprocess.PIPE,
-            stderr=subprocess.PIPE,
-            cwd=cwd,
-        )
-        graph_process.lastGraphTime = time.time()
-    elif (time.time() - graph_process.lastGraphTime) > cache.graphGenerateTime:
-        # graph_process.child = subprocess.Popen([sys.executable, 'generate_graphs.py', str(cache.json_path), '1', str(getCoreCounts())],
-        #    stdin=None, stdout=None, stderr=None, close_fds=True,  cwd=cwd)
-        subprocess.run(
-            [sys.executable, "generate_graphs.py", str(cache.json_path), str(cache.graphType), str(getCoreCounts())],
-            stdin=None,
-            stdout=None,
-            stderr=None,
-            close_fds=True,
-            cwd=cwd,
-        )
-        graph_process.lastGraphTime = time.time()
-    else:
-        if (time.time() - graph_process.lastMetaTime) > cache.graphMetaTime:
-            # graph_process.child = subprocess.Popen([sys.executable, 'generate_graphs.py', str(cache.json_path), '0', str(getCoreCounts())],
-            #    stdin=None, stdout=None, stderr=None, close_fds=True,  cwd=cwd)
-            subprocess.run(
-                [sys.executable, "generate_graphs.py", str(cache.json_path), "0", str(getCoreCounts())],
-                stdin=None,
-                stdout=None,
-                stderr=None,
-                close_fds=True,
-                cwd=cwd,
-            )
-            graph_process.lastMetaTime = time.time()
-
-
-def log_subprocess(name, ret):
-    for line in ret.stdout.splitlines():
-        multiprocessing.get_logger().info("%s stdout: %s", name, line)
-
-    for line in ret.stderr.splitlines():
-        multiprocessing.get_logger().info("%s stderr: %s", name, line)
-
-
 def finish(cache):
-    graph_process(cache, "Last", last=True)
-
-    if cache.graphSpearman:
-        cwd = str(Path(__file__).parent)
-        ret = subprocess.run(
-            [sys.executable, "video_spearman.py", str(cache.json_path), str(getCoreCounts())],
-            stdout=subprocess.PIPE,
-            stderr=subprocess.PIPE,
-            cwd=cwd,
-        )
-        log_subprocess("video_spearman.py", ret)
+    sub.graph_process(cache, "Last", last=True)
+    sub.graph_spearman(cache)
 
 
 def find_outliers(data, lower_percent=10, upper_percent=90):
@@ -1123,11 +1231,15 @@ def test_eta(eta, xl, xu, size):
     delta_q = numpy.zeros(size)
 
     xy[rand < 0.5] = 1.0 - delta_1
-    val[rand < 0.5] = 2.0 * rand[rand < 0.5] + (1.0 - 2.0 * rand[rand < 0.5]) * xy[rand < 0.5] ** (eta + 1)
+    val[rand < 0.5] = 2.0 * rand[rand < 0.5] + (1.0 - 2.0 * rand[rand < 0.5]) * xy[
+        rand < 0.5
+    ] ** (eta + 1)
     delta_q[rand < 0.5] = val[rand < 0.5] ** mut_pow - 1.0
 
     xy[rand >= 0.5] = 1.0 - delta_2
-    val[rand >= 0.5] = 2.0 * (1.0 - rand[rand >= 0.5]) + 2.0 * (rand[rand >= 0.5] - 0.5) * xy[rand >= 0.5] ** (eta + 1)
+    val[rand >= 0.5] = 2.0 * (1.0 - rand[rand >= 0.5]) + 2.0 * (
+        rand[rand >= 0.5] - 0.5
+    ) * xy[rand >= 0.5] ** (eta + 1)
     delta_q[rand >= 0.5] = 1.0 - val[rand >= 0.5] ** mut_pow
 
     delta_q = delta_q * (xu - xl)
@@ -1158,6 +1270,7 @@ def setupSimulation(sim, times, name, cache):
 
     sim.root.input.solver.user_solution_times = times
     sim.root.input.solver.sections.section_times[-1] = times[-1]
+    sim.root.input["return"].split_components_data = 1
 
     if cache.dynamicTolerance:
         sim.root.input.solver.time_integrator.abstol = get_evo_tolerance(cache, name)
@@ -1180,7 +1293,6 @@ def setupSimulation(sim, times, name, cache):
             sim.root.input["return"][unit].write_solution_solid = 0
             sim.root.input["return"][unit].write_solution_column_inlet = 0
             sim.root.input["return"][unit].write_solution_inlet = 0
-            sim.root.input["return"][unit].split_components_data = 0
 
             sim.root.input["return"][unit].write_sens_bulk = 0
             sim.root.input["return"][unit].write_sens_inlet = 0
@@ -1208,47 +1320,6 @@ def setupSimulation(sim, times, name, cache):
     sim.root.input.solver.nthreads = 1
 
 
-def graph_corner_process(cache, last=False, interval=1200):
-    if "last_time" not in graph_corner_process.__dict__:
-        graph_corner_process.last_time = time.time()
-
-    if "child" in graph_corner_process.__dict__:
-        if graph_corner_process.child.poll() is None:  # This is false if the child has completed
-            if last:
-                graph_corner_process.child.wait()
-            else:
-                return
-
-    cwd = str(Path(__file__).parent)
-
-    graph_scripts = ("generate_corner_graphs.py", "generate_autocorr_graphs.py", "generate_mixing_graphs.py")
-
-    if last:
-        for script in graph_scripts:
-            ret = subprocess.run(
-                [sys.executable, script, str(cache.json_path), str(getCoreCounts())],
-                stdin=None,
-                stdout=None,
-                stderr=None,
-                close_fds=True,
-                cwd=cwd,
-            )
-
-        graph_corner_process.last_time = time.time()
-    elif (time.time() - graph_corner_process.last_time) > interval:
-        for script in graph_scripts:
-            subprocess.run(
-                [sys.executable, script, str(cache.json_path), str(getCoreCounts())],
-                stdin=None,
-                stdout=None,
-                stderr=None,
-                close_fds=True,
-                cwd=cwd,
-            )
-
-        graph_corner_process.last_time = time.time()
-
-
 def biasSimulation(simulation, experiment, cache):
 
     bias_simulation = Cadet(simulation.root)
@@ -1257,7 +1328,7 @@ def biasSimulation(simulation, experiment, cache):
         name = experiment["name"]
 
         error_model = None
-        for error in cache.settings["kde_synthetic"]:
+        for error in cache.settings["errorModel"]:
             if error["name"] == name:
                 error_model = error
                 break
@@ -1288,7 +1359,9 @@ def setupLog(log_directory, log_name):
     logger.propagate = False
     logger.setLevel(logging.INFO)
 
-    logFormatter = logging.Formatter("%(asctime)s %(filename)s %(funcName)s %(lineno)d %(message)s")
+    logFormatter = logging.Formatter(
+        "%(asctime)s %(filename)s %(funcName)s %(lineno)d %(message)s"
+    )
 
     if not logger.handlers:
         stream = logging.StreamHandler(sys.stdout)
@@ -1310,7 +1383,11 @@ def setupLog(log_directory, log_name):
 
 
 def getCoreCounts():
-    cpus = int(sys.argv[-1])
+    try:
+        cpus = int(sys.argv[-1])
+    except ValueError:
+        # This happens when running in jupyter notebook or if the number can't be found, in either way default to just use availab cpu
+        cpus = None
     if cpus:
         return cpus
     else:
@@ -1325,11 +1402,14 @@ def getMapFunction():
 
         return map
     else:
-        pool = multiprocessing.Pool(cores)
+        if "pool" not in getMapFunction.__dict__:
+            pool = multiprocessing.Pool(cores)
+            getMapFunction.pool = pool
+            multiprocessing.get_logger().info(
+                "CADETMatch startup: created a parallel pool of %s workers", cores
+            )
 
-        multiprocessing.get_logger().info("CADETMatch startup: created a parallel pool of %s workers", cores)
-
-        return pool.imap_unordered
+        return getMapFunction.pool.imap_unordered
 
 
 def create_lookup(seq):
@@ -1372,13 +1452,16 @@ def fractionate_sim(start_times, stop_times, components, simulation, unit):
 
     fracs = {}
     for component in components:
-        sim_value = simulation.root.output.solution[unit]["solution_outlet_comp_%03d" % component]
+        sim_value = simulation.root.output.solution[unit][
+            "solution_outlet_comp_%03d" % component
+        ]
         spline = scipy.interpolate.InterpolatedUnivariateSpline(times, sim_value, ext=1)
 
         fracs[component] = fractionate_spline(start_times, stop_times, spline)
 
     return fracs
 
+
 def translate_meta_min(score, cache):
     temp = numpy.array(score)
     if cache.allScoreSSE:
@@ -1386,4 +1469,4 @@ def translate_meta_min(score, cache):
     else:
         temp[:3] = 1 - temp[:3]
 
-    return temp
\ No newline at end of file
+    return temp
